/**
 * AZARDI READER AND AZARDI INTERACTIVE ENGINE EXTENSIONS
 * Copyright 2007-2012 Infogrid Pacific Pte. Ltd. All rights reserved
 * Modified Date: 2012-04-25
 * 
 * @version: 1-2012-04-25
 * @author Anup Dhabarde
 */

/**
 * @class Contains functions necessary for creating, updating, saving and displaying notes.
 * This will be used in ACF. This will be dependent on Rangy latest version
 */
var Notes_ACF = function(){
	var config = {
		notesData: '',
		allNotesData: '',
		allNotesACFData: {},
		prompts: Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService),
		XHTML_NS: "http://www.w3.org/1999/xhtml"
	}
	
	/**
	 * Tests if the selected text is valid to create notes
	 * @returns {boolean} Valid/Invalid to create notes
	 */
	var isTextSelected = function(){
		var self = this;
		
		var notParagraph = false;
		
		var selectionObj = document.getElementById('doc-viewer-azd').contentWindow.getSelection();
		if(selectionObj.rangeCount > 1){
			var msg = getPropertiesString("azd_notes_mulSelNoteError");
			config.prompts.alert(null, AME.getTitle(),msg);
			// alert('Can\'t create note on multiple selections.');
			return false;
		}
		else if(selectionObj.toString() == ''){
			var msg = getPropertiesString("azd_notes_noTextSelNoteError");
			config.prompts.alert(null, AME.getTitle(), msg);
			// alert('Select some text to create note.');
			return false;
		}
		else if(selectionObj.toString().length < 6){
			var msg = getPropertiesString("azd_notes_minTextSelNoteError");
			config.prompts.alert(null, AME.getTitle(), msg);
			// alert('Select at least 6 characters.');
			return false;
		}
		else{
			var startNode = selectionObj.anchorNode;
			while(typeof startNode.tagName == 'undefined' || startNode.tagName.toLowerCase() != 'p' ){
				startNode = startNode.parentNode;
				if(startNode.tagName.toLowerCase() == 'body'){
					notParagraph = true;
					break;
				}
			}
			
			var endNode = selectionObj.focusNode;
			while(typeof endNode.tagName == 'undefined' || endNode.tagName.toLowerCase() != 'p'){
				endNode = endNode.parentNode;
				if(endNode.tagName.toLowerCase() == 'body'){
					notParagraph = true;
					break;
				}
			}
			
			if(notParagraph){
				var msg = getPropertiesString("azd_notes_noParaNoteError");
				config.prompts.alert(null, AME.getTitle(), msg);
				// alert('Can\'t create note here.');
				return false;
			}
			else if(startNode.getAttribute('class') != endNode.getAttribute('class')){
				var msg = getPropertiesString("azd_notes_mulParaSelectNoteError");
				config.prompts.alert(null, AME.getTitle(), msg);
				// alert('Select only one paragraph for creating note.');
				return false;
			}
		}
		
//		var selectedPara = self.getSelectedParagraph();
//		var paraContent = $('#doc-viewer-azd').contents().find('.'+selectedPara).html();
//		if(paraContent.search(selectionObj.toString()) == -1){
//			alert('5 Can\'t create note here.');
//			return false;
//		}
		
		return true;
	}
	
//	var getSelectedParagraph = function(){
//		var selectionObj = document.getElementById('doc-viewer-azd').contentWindow.getSelection();
//		
//		var startNode = selectionObj.anchorNode;
////		alert(selectionObj.anchorNode.text())
//		while(typeof startNode.tagName == 'undefined' || startNode.tagName.toLowerCase() != 'p' ){
//			startNode = startNode.parentNode;
//			if(startNode.tagName.toLowerCase() == 'body'){
//				break;
//			}
//		}
//		
//		var className = '';
//		var classes = startNode.getAttribute('class').split(' ');
//		for(var i = 0; i < classes.length; i++){
//			if(classes[i].search('azd-notes-paragraph-') != -1){
//				className = classes[i];
//				break;
//			}
//		}
//		
//		return className;
//	}
	
//	var getStartOffset = function(paragraphClass){
//		var selectionObj = document.getElementById('doc-viewer-azd').contentWindow.getSelection();
//		var paragraphContent = $('#doc-viewer-azd').contents().find('.'+paragraphClass).html();
//		var selectionObj = selectionObj.toString().replace(/\n/g, ' ');
//		var startOffset = paragraphContent.search(selectionObj);
//		
//		return startOffset;
//	}
//	
//	var getSelectionLength = function(){
//		var selectionObj = document.getElementById('doc-viewer-azd').contentWindow.getSelection();
//		
//		return selectionObj.toString().length;
//	}
	
	/**
	 * Returns the selected string
	 * @returns {string} selected string
	 */
	var getSelectedString = function(){
		var selectionObj = document.getElementById('doc-viewer-azd').contentWindow.getSelection();
		var selRange = selectionObj.getRangeAt(0);
		
		var selection = selectionObj.toString();
		if(selection.length > 170){
			selection = selection.substr(0, 168) + '...';
		}
		
		return selection;
	};
	
	
	var createACFSNotesDetail = function(bookId, currentChapter, selContent, serialized_data, currentTime, userContent) {
		var shelfdisp = Shelf.getPubShelfDisplayInfo(Shelf.config.currentShelfId);
		var agreement_id = "";
		var rendition_layout = "";
		var fixlayout = false;
		var section_id = Epub.getCurrentChapterFromSpine();
		for(var i = 0; i < Library.config.bookMetadata.length; i++){
			if("" + Library.config.bookMetadata[i]['workuid'] === bookId){
				agreement_id = Library.config.bookMetadata[i]["workuid"];
				rendition_layout = Library.config.bookMetadata[i]["rendition_layout"];
				if (rendition_layout != "reflowable") fixlayout = true;
				break;
			}
		}
		
		var store_token = shelfdisp["shelf_md"]["store_token"];
        var device_token = shelfdisp["shelf_md"]["devicetoken"];
		var customer_id = shelfdisp["shelf_md"]["user_id"];
        
        var data_type =0;
        var note_guid = "igp-note-" + Math.random().toString().split(".")[1];
        var highlight_name_chaptitle = document.getElementById('doc-viewer-azd').contentWindow.wrappedJSObject.getSectionHeading();
        var real_page_no =  "";
	    var standalone_page = false;
		
		return {
			"store_token":store_token,
			"device_token":device_token,
			"agreement_id":agreement_id,
			"customer_id":customer_id,
			"data_type":data_type,
			"highlight_name":highlight_name_chaptitle,
			"real_page_no":real_page_no,
			"standalone_page":standalone_page,
			"guid" : note_guid,
			"fixlayout":fixlayout,	
			"section_id":section_id,	
			"highlight_content": selContent,
			"serialized_data": serialized_data,
            "selection_node_json" : serialized_data,
            "user_data" : userContent,
			"created_date" : currentTime
		}
		
	}
	
	/**
	 * Saves the notes data to database
	 * @params bookId Id of the book
	 * @params currentChapter Current chapter of the book
	 * @params notesDescription Description of the notes
	 * @params selectionString Selected string
	 * @params currentTime Current time
	 * @params serial Serialized values of notes
	 * @params noteId Id of the note
	 * @params edit new save | edited save
	 */
	var save = function(bookId, currentChapter, notesDescription, selectionString, currentTime, serial, noteId, edit, ishilight){
		var fileObj = getFileObject(AME.getDbFilePath());
		var obj = createACFSNotesDetail(bookId, currentChapter, selectionString, serial, currentTime, notesDescription);
		
		var table_name = 'annotation';
		var data_type = "0";
		var userdata_type = 'NOTE';
		if (ishilight) {
			userdata_type = 'HILIGHT';
			data_type = "1";
		}
		
		if(edit){
			var conditions = new Array(['id', noteId, 'INTEGER']);
			var values = new Array(['notes_description', notesDescription, 'TEXT'],
			  					   ['time_created', currentTime, 'TEXT']);
			updateRowIntoDatabase(fileObj, values, table_name, conditions);
			
			var table_name = 'userdata_acf';
			var conditions = new Array(['annotation_id', noteId, 'TEXT']);
			var values = new Array(['user_data', obj.user_data, 'TEXT'],
			  					    ['syncstatus', false, 'INTEGER']);
			updateRowIntoDatabase(fileObj, values, table_name, conditions);
			
		}
		else{
			var values = new Array(['book_id', bookId, 'TEXT'],
								   ['current_chapter', currentChapter, 'TEXT'],
								   ['notes_title', userdata_type, 'TEXT'], //We are going to use this to store the type as this field is not used for anything else
								   ['notes_description', notesDescription, 'TEXT'],
						  		   ['selection_string', selectionString, 'TEXT'],
								   ['serial', serial, 'TEXT'],
								   ['time_created', currentTime, 'TEXT']);
			var noteid = insertRowIntoDatabase(fileObj, values, table_name);
			
			var values = new Array(['annotation_id', "" + noteid, 'TEXT'],
								   ['store_token', obj.store_token, 'TEXT'],
								   ['device_token', obj.device_token, 'TEXT'],
								   ['agreement_id', obj.agreement_id, 'TEXT'],
						  		   ['customer_id', obj.customer_id, 'TEXT'],
								   ['data_type', data_type, 'TEXT'],
								   ['real_page_no', obj.real_page_no, 'TEXT'],
								   ['standalone_page', obj.standalone_page, 'INTEGER'],
								   ['guid', obj.guid, 'TEXT'],
								   ['fixlayout', obj.fixlayout, 'INTEGER'],
								   ['section_id', obj.section_id, 'TEXT'],
								   ['section_name', obj.highlight_name, 'TEXT'],
								   ['highlight_content', obj.highlight_content, 'TEXT'],
								   ['user_data', obj.user_data, 'TEXT'],
								   ['serialized_data', obj.serialized_data, 'TEXT'],
								   ['selection_node_json', obj.selection_node_json, 'TEXT'],
								   ['created_date', obj.created_date, 'TEXT'],
								   ['syncstatus', false, 'INTEGER']);
								   
			var table_name = 'userdata_acf';
			insertRowIntoDatabase(fileObj, values, table_name);
		}
	}
	
	/**
	 * Retrieve all the notes of current chapter from database
	 */
	function getNotes(bookId, currentChapter){
		var fileObj = getFileObject(AME.getDbFilePath());
	    if(!fileObj.exists()){
	        return false;
	    }
		
		var table_name = 'annotation';
		var condition = new Array(['book_id', bookId, 'TEXT'],
								  ['current_chapter', currentChapter, 'TEXT']);
//		var values = ['serial', 'notes_title', 'notes_description', 'selection_string', 'time_created', 'id'];
		var values = ['serial', 'notes_description', 'selection_string', 'time_created', 'id', 'notes_title'];
		config.notesData = new Array();
		statement = getValuesFromDatabase(fileObj, values, table_name, condition);
		
		if(statement !== undefined && statement !== false){
			while (statement.executeStep()){
				var data = new Array();
				data.push(statement.row.serial);
//				data.push(statement.row.notes_title);
				data.push(statement.row.notes_description);
				data.push(statement.row.selection_string);
				data.push(statement.row.time_created);
				data.push(statement.row.id);
				//We will use this field to separate Notes and highlight. if it is empty
				//it is a note. Else allowed values are NOTE|HIGHLIGHT
				data.push(statement.row.notes_title); 
				config.notesData.push(data);
		    }
		    statement.finalize();
		}
	};
	
	/**
	 * Get all the notes data from database
	 * @params bookId Id of the current book
	 */
	var getAllNotes = function(bookId){
		var fileObj = getFileObject(AME.getDbFilePath());
	    if(!fileObj.exists()){
	        return false;
	    }
	    var table_name = 'annotation';
		var condition = new Array(['book_id', bookId, 'TEXT']);
//		var values = ['serial', 'notes_title', 'notes_description', 'selection_string', 'time_created', 'id'];
		var values = ['serial', 'notes_description', 'selection_string', 'time_created', 'id', 'current_chapter', 'notes_title'];
		config.allNotesData = new Array();
		statement = getValuesFromDatabase(fileObj, values, table_name, condition);
		
		if(statement !== undefined && statement !== false){
			while (statement.executeStep()){
				var data = new Array();
				data.push(statement.row.serial);
				data.push(statement.row.notes_description);
				data.push(statement.row.selection_string);
				data.push(statement.row.time_created);
				data.push(statement.row.id);
				data.push(statement.row.current_chapter);
				data.push(statement.row.notes_title);
				
				config.allNotesData.push(data);
		    }
		    statement.finalize();
		}
	}
	
	var getUserDataType = function (note_title) {
		if (!note_title) {
			return "NOTE";
		}
		
		return note_title;
	}
	
	var getSelectionSerializationType = function (serial_string) {
		if (serial_string[0] == "{") {
			return "IND";
		}
		
		return "NATIVE";
	}
	
	/**
	 * Highlight the notes in the current page
	 */
	var highlight = function(){
		for(var i = 0; i < config.notesData.length; i++){
			var serial = config.notesData[i][0],
				   noteTitle = config.notesData[i][5],
				   noteDescription = config.notesData[i][1],
				   noteTime = config.notesData[i][3],
				   noteId = config.notesData[i][4];
			var ishilight = true;
			
			if(noteTitle == "NOTE"){
				ishilight = false;
			}
			
			try{
				var sel = document.getElementById('doc-viewer-azd').contentWindow.wrappedJSObject.IGP_RANGE.setSelectionInfo(serial);
				document.getElementById('doc-viewer-azd').contentWindow.wrappedJSObject.rangyHighlightSelectedText(noteId, ishilight);				
			}
			catch(e){
				alert("ACF highlight" + e);
			}
		}
	};
	
	/**
	 * Highlights selected notes string on the fly
	 * @params serial Serialized value of notes
	 * @params noteId Id of the note
	 */
	var highlightOnTheFly = function(serial, noteId, ishilight){
		try {
			var sel = document.getElementById('doc-viewer-azd').contentWindow.wrappedJSObject.IGP_RANGE.setSelectionInfo(serial);
			document.getElementById('doc-viewer-azd').contentWindow.wrappedJSObject.rangyHighlightSelectedText(noteId, ishilight);
			return;
		} catch (e) {
			alert("highlightOnTheFly >>>" + e);
		}
	}
	
	/**
	 * Creates the edit note dialog
	 */
	var createEditDialog = function(){
		document.getElementById('doc-viewer-azd').contentWindow.wrappedJSObject.createNoteEditDialog();
	};
	
	/**
	 * Fills the notes information in the note dialog
	 * @params noteId Id of the note to be edited
	 */
	var setEditDialog = function(noteId){
		var dateObj = new Date();
		
		for(var i = 0; i < config.notesData.length; i++){
			if(noteId == config.notesData[i][4]){
				dateObj.setTime(config.notesData[i][3]);
				var shortTitle = config.notesData[i][2];
				if(shortTitle.length > 27){
					var shortTitle  = shortTitle.substring(0, 25) + '...';
				}
				$('#doc-viewer-azd').contents().find('.notes-title-ape').html(shortTitle);
				var noteDescription = config.notesData[i][1].replace( /\n/g, '<br/>' );
				$('#doc-viewer-azd').contents().find('.notes-description-ape').html(noteDescription);
				$('#doc-viewer-azd').contents().find('.notes-date-ape').html(dateObj.toDateString());
				$('#doc-viewer-azd').contents().find('#id_anno').html(noteId);
			}
		}
	};
	
	/**
	 * Returns the id of the latest note added to database
	 * @returns {string} Id of the latest note
	 */
	var getLatestId = function(){
		var fileObj = getFileObject(AME.getDbFilePath());
	    if (!fileObj.exists()){
	        return false;
	    }
	    var table_name = 'annotation';
		var values = ['id'];
		statement = getValuesFromDatabase(fileObj, values, table_name);
		var noteId = new Array();
		if(typeof statement !== 'undefined' && statement !== false){
			while (statement.executeStep()){
				noteId.push(statement.row.id);
		    }
		    statement.finalize();
		}
		return noteId.pop();
	}
	
	/**
	 * Adds the new note to array for quick reference
	 * @params notesDescription Description of notes
	 * @params selectionString String selected
	 * @params serial Serialized value of notes
	 * @params currentTime Current time
	 * @params noteId Id of the note
	 */
	var addToArray = function(notesDescription, selectionString, serial, currentTime, noteId, ishilight){
		try {
		var data = new Array();
		data.push(serial);
//		data.push(notesTitle);
		data.push(notesDescription);
		data.push(selectionString);
		data.push(currentTime);
		data.push(noteId);
		data.push(ishilight);
		
		config.notesData.push(data);
		} catch(e) {
			alert(e);
		}
	};
	
	var updateArray = function(notesDescription, currentTime, noteId){
		for(var i = 0; i < config.notesData.length; i++){
			if(noteId == config.notesData[i][4]){
				config.notesData[i][1] = notesDescription;
				config.notesData[i][3] = currentTime;
				break;
			}
		}
	}
	
//		<ul id="note-container">
//			<li id="notes-item">
//				<h4><span id="notes-section-title">Chapter 1:: Tiger Tiger</span> <span id="notes-date">Thu Jun 16 2011</span> <span class="delete_note">Delete</span></h4>
//				<p id="notes-highlight">for discussions over the past couple years about feature detection and particularly to David Mark for the long discussions over the past three months. David's code has stood the scrutiny of the group and are </p>
//		    </li>
//		</ul>
	
	/**
	 * Create and append all notes page
	 */
	var createAllNotesPage = function(){
		try{
			var dateObj = new Date();
			
			for(var i = 0; i < config.allNotesData.length; i++){
				dateObj.setTime(config.allNotesData[i][3]);
				if (config.allNotesData[i][6] == "HILIGHT") {
					continue;
				}
				
				var shortDescription = config.allNotesData[i][1];
				if(shortDescription.length > 20){
					shortDescription = shortDescription.substring(0, 18) + '...';
				}
				
				var notesPage = $(document.getElementById('doc-viewer-azd').contentWindow.document.createElement('ul'))
								  .attr('id', 'note-container')
								  .data('id', config.allNotesData[i][4])
								  .data('chapter', config.allNotesData[i][5])
								  .append($(document.getElementById('doc-viewer-azd').contentWindow.document.createElement('li'))
							  			.attr('id', 'notes-item')
										.append($(document.getElementById('doc-viewer-azd').contentWindow.document.createElement('h4'))
											.append($(document.getElementById('doc-viewer-azd').contentWindow.document.createElement('span'))
												.attr('id', 'notes-section-title')
												.click(function(){
													AME.goToNote($(this).parent().parent().parent().data('id'), $(this).parent().parent().parent().data('chapter'));
												})
												.append($(document.getElementById('doc-viewer-azd').contentWindow.document.createTextNode(shortDescription)))
											)
											.append($(document.getElementById('doc-viewer-azd').contentWindow.document.createElement('span'))
												.attr('id', 'notes-date')
												.click(function(){
													AME.goToNote($(this).parent().parent().parent().data('id'), $(this).parent().parent().parent().data('chapter'));
												})
												.append($(document.getElementById('doc-viewer-azd').contentWindow.document.createTextNode(dateObj.toDateString())))
											)
											.append($(document.getElementById('doc-viewer-azd').contentWindow.document.createElement('span'))
												.addClass('delete_note')
												.click(function(){
													AME.deleteNote($(this).parent().parent().parent().data('id'), this);
												})
												.append($(document.getElementById('doc-viewer-azd').contentWindow.document.createTextNode('Delete')))
											)
										)
										.append($(document.getElementById('doc-viewer-azd').contentWindow.document.createElement('p'))
											.attr('id', 'notes-highlight')
											.click(function(){
												AME.goToNote($(this).parent().parent().data('id'), $(this).parent().parent().data('chapter'));
											})
											.append($(document.getElementById('doc-viewer-azd').contentWindow.document.createTextNode(config.allNotesData[i][2])))
										)
								   )
				$('#doc-viewer-azd').contents().find('#notes_placeholder').append(notesPage);
			}
		}catch(e){alert(e)}
	}
		/**
	 * Create and append all notes page
	 */
	var createAllHilitesPage = function(){
		try{
			var dateObj = new Date();
			
			for(var i = 0; i < config.allNotesData.length; i++){
				dateObj.setTime(config.allNotesData[i][3]);
				if (config.allNotesData[i][6] == "NOTE" || config.allNotesData[i][6] == "" ) {
					continue;
				}
				
				var shortDescription = config.allNotesData[i][1];
				
				var hlPage = $(document.getElementById('doc-viewer-azd').contentWindow.document.createElement('ul'))
								  .attr('id', 'hilite-container')
								  .data('id', config.allNotesData[i][4])
								  .data('chapter', config.allNotesData[i][5])
								  .append($(document.getElementById('doc-viewer-azd').contentWindow.document.createElement('li'))
							  			.attr('id', 'notes-item')
										.append($(document.getElementById('doc-viewer-azd').contentWindow.document.createElement('h4'))
											.append($(document.getElementById('doc-viewer-azd').contentWindow.document.createElement('span'))
												.attr('id', 'notes-section-title')
												.click(function(){
													AME.goToNote($(this).parent().parent().parent().data('id'), $(this).parent().parent().parent().data('chapter'), true);
												})
												.append($(document.getElementById('doc-viewer-azd').contentWindow.document.createTextNode(shortDescription)))
											)
											.append($(document.getElementById('doc-viewer-azd').contentWindow.document.createElement('span'))
												.attr('id', 'notes-date')
												.click(function(){
													AME.goToNote($(this).parent().parent().parent().data('id'), $(this).parent().parent().parent().data('chapter'), true);
												})
												.append($(document.getElementById('doc-viewer-azd').contentWindow.document.createTextNode(config.allNotesData[i][2] /*+ dateObj.toDateString()*/)))
											)
											.append($(document.getElementById('doc-viewer-azd').contentWindow.document.createElement('span'))
												.addClass('delete_note')
												.click(function(){
													AME.deleteNote($(this).parent().parent().parent().data('id'), this);
												})
												.append($(document.getElementById('doc-viewer-azd').contentWindow.document.createTextNode('Delete')))
											)
										)
										.append($(document.getElementById('doc-viewer-azd').contentWindow.document.createElement('p'))
											.attr('id', 'notes-highlight')
											.click(function(){
												AME.goToNote($(this).parent().parent().data('id'), $(this).parent().parent().data('chapter'), true);
											})
											//.append($(document.getElementById('doc-viewer-azd').contentWindow.document.createTextNode(config.allNotesData[i][2])))
										)
								   )
				$('#doc-viewer-azd').contents().find('#hilites_placeholder').append(hlPage);
			}
		}catch(e){alert(e)}
	}
	
	/**
	 * Sort all the notes data depending upon the date added
	 */
	var sortAllNotes = function(){
		var notesSort = function(a, b){
			if(parseInt(a[3]) < parseInt(b[3]))
	            return 1;
	        else if(parseInt(a[3]) > parseInt(b[3]))
	        	return -1;
	        else
	        	return 0;
	    }
		
		config.allNotesData.sort(notesSort);
	};
	
	/**
	 * 	Deletes the note from the database
	 * @params noteId Id of the note to be deleted
	 */
	var deleteNoteFromDatabase = function(noteId){
		var fileObj = getFileObject(AME.getDbFilePath());
		var table_name = 'annotation';
		var conditions = new Array(['id', noteId, 'INTEGER']);
		deleteRowFromDatabase(fileObj, table_name, conditions);
		
		var table_name = 'userdata_acf';
		var conditions = new Array(['annotation_id', noteId, 'TEXT']);
		deleteRowFromDatabase(fileObj, table_name, conditions);
		
	};
	
	/**
	 * Deletes all the notes for this book from database
	 * @params bookId Id of the book to be deleted
	 */
	var deleteAllNotesFromDatabase = function(bookId){
		var fileObj = getFileObject(AME.getDbFilePath());
		var table_name = 'annotation';
		var conditions = new Array(['book_id', bookId, 'TEXT']);
		deleteRowFromDatabase(fileObj, table_name, conditions);
		
		var table_name = 'userdata_acf';
		var conditions = new Array(['agreement_id', bookId, 'TEXT']);
		deleteRowFromDatabase(fileObj, table_name, conditions);
		
	}
	
	/**
	 * Delete the note from the full page
	 */
	var deleteNoteFromFullPage = function(deleteButton){
		$(deleteButton).parent().parent().parent().remove();
	};
	
	var getBookSyncDetails = function(bookId) {
		var shelfdisp = Shelf.getPubShelfDisplayInfo(Shelf.config.currentShelfId);
		var agreement_id = "";
		
		for(var i = 0; i < Library.config.bookMetadata.length; i++){
			if("" + Library.config.bookMetadata[i]['workuid'] === bookId){
				agreement_id = Library.config.bookMetadata[i]["workuid"];
				rendition_layout = Library.config.bookMetadata[i]["rendition_layout"];
				if (rendition_layout != "reflowable") fixlayout = true;
				break;
			}
		}
		
		var store_token = shelfdisp["shelf_md"]["store_token"];
        var device_token = shelfdisp["shelf_md"]["devicetoken"];
		var customer_id = shelfdisp["shelf_md"]["user_id"];
		var domain_name = shelfdisp["shelf_md"]["domain_name"];
        
        var data_type =0;
       
		return {
			"store_token":store_token,
			"device_token":device_token,
			"agreement_id":agreement_id,
			"customer_id":customer_id,
			"domain_name":domain_name,
		}
		
	}
	
	
	var createACFSNotesShelfDetail = function(bookId) {
		var shelfdisp = Shelf.getPubShelfDisplayInfo(Shelf.config.currentShelfId);
		var agreement_id = bookId;
		var rendition_layout = "";
		var fixlayout = false;
		
		var store_token = shelfdisp["shelf_md"]["store_token"];
        var device_token = shelfdisp["shelf_md"]["devicetoken"];
		var customer_id = shelfdisp["shelf_md"]["user_id"];
        
		return {
			"store_token":store_token,
			"device_token":device_token,
			"agreement_id":agreement_id,
			"customer_id":customer_id
		}
		
	}
	
	/**
	 * Saves the notes data to database
	 * @params bookId Id of the book
	 * @params currentChapter Current chapter of the book
	 * @params notesDescription Description of the notes
	 * @params selectionString Selected string
	 * @params currentTime Current time
	 * @params serial Serialized values of notes
	 * @params noteId Id of the note
	 * @params edit new save | edited save
	 */
	var saveRemoteNote = function(bookId, currentChapterIDRef, syncdata, edit, noteId){
		try{
		var fileObj = getFileObject(AME.getDbFilePath());
		
		var shelfobj = createACFSNotesShelfDetail(bookId);
		
		var obj =  {
			"store_token":shelfobj.store_token,
			"device_token":shelfobj.device_token,
			"agreement_id":shelfobj.agreement_id,
			"customer_id":shelfobj.customer_id,
			"data_type":syncdata.data_type,
			"highlight_name":syncdata["highlight_name"],
			"real_page_no":syncdata["real_page_no"],
			"standalone_page":syncdata["standalone_page"],
			"guid" : syncdata["guid"],
			"fixlayout":syncdata["fixlayout"],
			"section_id":section_id,	
			"highlight_content": syncdata["highlight_content"],
			"serialized_data": syncdata["selection_node_json"],
            "selection_node_json" : syncdata["selection_node_json"],
            "user_data" : syncdata["note_text"],
			"created_date" : syncdata["created_date"]
		}
		
		var table_name = 'annotation';
		
		var notesDescription = obj.user_data;
		var selectionString = obj.highlight_content;
		var serial = obj.selection_node_json;
		var currentTime = obj.created_date;
		var currentChapter = Epub.getChapterIdFromSpineId(currentChapterIDRef);
		
		if(edit){
			var conditions = new Array(['id', parseInt(noteId), 'INTEGER']);
			var values = new Array(['notes_description', notesDescription, 'TEXT'],
			  					   ['time_created', currentTime, 'TEXT']);
			updateRowIntoDatabase(fileObj, values, table_name, conditions);
			
			var table_name = 'userdata_acf';
			var conditions = new Array(['annotation_id', noteId, 'TEXT']);
			var values = new Array(['user_data', obj.user_data, 'TEXT'],
			  					    ['syncstatus', true, 'INTEGER']);
			updateRowIntoDatabase(fileObj, values, table_name, conditions);
			
		}
		else{
			var userdata_type = 'NOTE';
			if (obj.data_type != 0) {
				userdata_type = 'HILIGHT';
			}
			
			if (obj.data_type == 0) {
				
			}
			
			var values = new Array(['book_id', bookId, 'TEXT'],
								   ['current_chapter', "" + currentChapter, 'TEXT'],
								   ['notes_title', userdata_type, 'TEXT'],
								   ['notes_description', notesDescription, 'TEXT'],
						  		   ['selection_string', selectionString, 'TEXT'],
								   ['serial', serial, 'TEXT'],
								   ['time_created', currentTime, 'TEXT']);
								   
			var noteid = insertRowIntoDatabase(fileObj, values, table_name);
			
			var values = new Array(['annotation_id', "" + noteid, 'TEXT'],
								   ['store_token', obj.store_token, 'TEXT'],
								   ['device_token', obj.device_token, 'TEXT'],
								   ['agreement_id', obj.agreement_id, 'TEXT'],
						  		   ['customer_id', obj.customer_id, 'TEXT'],
								   ['data_type', obj.data_type, 'TEXT'],
								   ['real_page_no', obj.real_page_no, 'TEXT'],
								   ['standalone_page', obj.standalone_page, 'INTEGER'],
								   ['guid', obj.guid, 'TEXT'],
								   ['fixlayout', obj.fixlayout, 'INTEGER'],
								   ['section_id', obj.section_id, 'TEXT'],
								   ['section_name', obj.highlight_name, 'TEXT'],
								   ['highlight_content', obj.highlight_content, 'TEXT'],
								   ['user_data', obj.user_data, 'TEXT'],
								   ['serialized_data', obj.serialized_data, 'TEXT'],
								   ['selection_node_json', obj.selection_node_json, 'TEXT'],
								   ['created_date', obj.created_date, 'TEXT'],
								   ['syncstatus', false, 'INTEGER']);
								   
			var table_name = 'userdata_acf';
			insertRowIntoDatabase(fileObj, values, table_name);
		}
		} catch (e) {
			alert("saveRemoteNote" + e);
		}
	}
	
	
	var loadAllACFNotesMetadata = function (bookId) {
		config.allNotesACFData = {};
		var fileObj = getFileObject(AME.getDbFilePath());
		
		var table_name = 'userdata_acf';
		var condition = new Array(['agreement_id', bookId, 'TEXT']);
		var values = ['annotation_id', 'store_token', 'device_token', 'customer_id', 'data_type', 'guid', 'section_id', 'user_data', 'serialized_data', 'syncstatus'];
		statement = getValuesFromDatabase(fileObj, values, table_name, condition);
		
		if(statement !== undefined && statement !== false){
			while (statement.executeStep()){
				var data = {};
				data["annotation_id"] = statement.row.annotation_id;
				data["store_token"] = statement.row.store_token;
				data["device_token"] = statement.row.device_token;
				data["customer_id"] = statement.row.customer_id;
				data["data_type"] = statement.row.data_type;
				data["guid"] = statement.row.guid;
				data["section_id"] = statement.row.section_id;
				data["user_data"] = statement.row.user_data;
				data["serialized_data"] = statement.row.serialized_data;
				data["syncstatus"] = statement.row.syncstatus;
				
				config.allNotesACFData[data["guid"]] = data;
		    }
		    statement.finalize();
		}
		
	}
	
	var saveSyncDataToLocal = function (bookId, sync_data) {
		for (section_id in sync_data) {
			var section_data = sync_data[section_id];
			for (var i=0; i<section_data.length; i++){
				var syncdata = section_data[i];
				var guid = syncdata["guid"];
				var noteId = "";
				var edit = false;
				if (config.allNotesACFData.hasOwnProperty(guid)){
					edit = true;
					noteId = config.allNotesACFData[guid]["annotation_id"];
				}
				try {
					saveRemoteNote(bookId, section_id, syncdata, edit, noteId);
				} catch(e) {
					alert("saveSyncDataToLocal" + e);
				}
			}
		}
	};
	
	var syncDownloadUserData = function (bookId) {
		var conninfo = getBookSyncDetails(bookId);
		var caturl = conninfo["domain_name"] + "/acfs/newapi/syncuserdata/";
		var paramData = "store_token=" + conninfo.store_token + "&device_token=" + conninfo.device_token + "&agreement_id=" + conninfo.agreement_id + "&data_type=0";
        var url = caturl +  "?" + paramData;
		
		loadAllACFNotesMetadata(bookId);
		
		/*var last_sync_info = getLastSyncDate(conninfo.device_token, conninfo.agreement_id, "DOWNLOAD");
		var data_to_be_sync = createSyncDataForUpload(bookId);
		alert(JSON.stringify(data_to_be_sync));
		return;
		
		$.getJSON(url, function(sync_data){
			if (sync_data.status && sync_data.user_data_count>0) {
				saveSyncDataToLocal(bookId, sync_data["data"]);
			}
			
			var sync_pk = "";
			var insert = true;
			if (last_sync_info.hasOwnProperty("id")) {
				sync_pk = last_sync_info["id"];
				insert = false;
			}
			
			setLastSyncStatus(sync_pk, conninfo.device_token, conninfo.agreement_id, "DOWNLOAD", "1", "SUCCESS", insert, sync_pk)
		});*/
	};
	
	var getLastSyncDate = function (devicetoken, bookId, synctype) {
		var fileObj = getFileObject(AME.getDbFilePath());
		
		var table_name = 'userdata_sync_acf';
		alert();
		var conditions = new Array(['agreement_id', bookId, 'TEXT'], ['device_token', devicetoken, 'TEXT'], ['sync_type', synctype, 'TEXT']);
		var values = ['id', 'sync_date', 'sync_status', 'sync_message'];
		statement = getValuesFromDatabase(fileObj, values, table_name, conditions);
		var res = {};
		if(statement !== undefined && statement !== false){
			while (statement.executeStep()){
				var data = {};
				data["id"] = statement.row.id;
				data["device_token"] = devicetoken;
				data["sync_status"] = statement.row.sync_status;
				data["sync_message"] = statement.row.sync_message;
				data["agreement_id"] = bookId;
				data["sync_date"] = statement.row.sync_date;
			
				res = data;
				break;
		    }
		    statement.finalize();
		}
		return res;
	};
	
	var setLastSyncStatus = function (sync_pk, device_token, agreement_id, sync_type, sync_status, sync_message, insert) {
		var fileObj = getFileObject(AME.getDbFilePath());
		
		var sync_date = Date.now();
		var table_name = 'userdata_sync_acf';
		
		if (insert) {
			var values = new Array(['device_token', device_token, 'TEXT'],
								   ['agreement_id', "" + agreement_id, 'TEXT'],
								   ['sync_type', sync_type, 'TEXT'],
						  		   ['sync_status', sync_status, 'TEXT'],
								   ['sync_message', sync_message, 'TEXT'],
								   ['sync_date', sync_date, 'TEXT']);
								   
			insertRowIntoDatabase(fileObj, values, table_name);
		} else {
			var conditions = new Array(['id', sync_pk, 'INTEGER']);
			var values = new Array(['sync_date', sync_date, 'TEXT'], ['sync_status', sync_status, 'TEXT'], ['sync_message', sync_status, 'TEXT']);
			updateRowIntoDatabase(fileObj, values, table_name, conditions);
		}
		
	};
	
	var createSyncDataForUpload = function(bookId) {
		try {
			var fileObj = getFileObject(AME.getDbFilePath());
			
			var table_name = 'userdata_acf';
			
			var conditions = new Array(['agreement_id', bookId, 'TEXT'], ['syncstatus', 0, 'INTEGER']);
			var values = ["store_token" , "device_token", "agreement_id", "customer_id", "data_type", "section_name", "real_page_no", "standalone_page" , "guid", "fixlayout", "section_id", "highlight_content", "user_data", "serialized_data", "selection_node_json", "created_date"];
			statement = getValuesFromDatabase(fileObj, values, table_name, conditions);
			var res = {};
			if(statement !== undefined && statement !== false){
				while (statement.executeStep()){
					var obj =  {
						"store_token":statement.row.store_token,
						"device_token":statement.row.device_token,
						"agreement_id":statement.row.agreement_id,
						"customer_id":statement.row.customer_id,
						"data_type":statement.row.data_type,
						"highlight_name":statement.row.section_name,
						"real_page_no":statement.row.real_page_no,
						"standalone_page":statement.row.standalone_page,
						"guid" : statement.row.guid,
						"fixlayout":statement.row.fixlayout,
						"highlight_content": statement.row.highlight_content,
						"serialized_data": statement.row.selection_node_json,
						"selection_node_json" : statement.row.selection_node_json,
						"note_text" : statement.row.user_data,
						"created_date" : statement.row.created_date
					}
					var section_id = statement.row.section_id;
					res[section_id] = obj;
				}
				statement.finalize();
			}
			return res;
		} catch(e) {
			alert(e);
		}
	}
	
	return{
		config: config,
		isTextSelected: isTextSelected,
//		getSelectedParagraph: getSelectedParagraph,
//		getStartOffset: getStartOffset,
//		getSelectionLength: getSelectionLength,
		createEditDialog: createEditDialog,
		getSelectedString: getSelectedString,
		save: save,
		getNotes: getNotes,
		getAllNotes: getAllNotes,
		highlight: highlight,
		highlightOnTheFly: highlightOnTheFly,
		setEditDialog: setEditDialog,
		getLatestId: getLatestId,
		addToArray: addToArray,
		updateArray: updateArray,
		createAllNotesPage: createAllNotesPage,
		createAllHilitesPage: createAllHilitesPage,
		sortAllNotes: sortAllNotes,
		deleteNoteFromDatabase: deleteNoteFromDatabase,
		deleteAllNotesFromDatabase: deleteAllNotesFromDatabase,
		deleteNoteFromFullPage: deleteNoteFromFullPage,
		syncDownloadUserData: syncDownloadUserData
	}
}();